/*
** Module   : p2j.js
** Abstract : top-level web client java script object 
**
** Copyright (c) 2014-2015, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**         Golden Code Development Corporation
**                 CONFIDENTIAL
**
** -#- -I- --Date-- ------------------------------Description----------------------------------
** 001 MAG 20140110 First version.
** 002 MAG 20140204 Prevent back navigation.  Disable browser context menu.
** 003 MAG 20140702 Add tty.clipboard module.
** 004 GES 20150324 Moved this class to a common resource base and made it generic (so that it
**                  can be used for both ChUI and GUI web clients.
*/

/**
 * Create a top-level JavaScript object (which acts as a namespace for the web client code)
 * and initialize all submodules.
 * <p>
 * The following order of initialization is required:
 * <p>
 * <ol>
 *    <li> p2j.sound
 *    <li> p2j.screen
 *    <li> p2j.socket
 *    <li> p2j.keyboard
 *    <li> p2j.clipboard_helpers
 *    <li> p2j.clipboard
 * </ol>
 */
p2j = (function()
{
   /** The top-level web client object which will be returned from this function. */
   var me = {};

   /** OS detection flags. */
   me.isWindows = navigator.appVersion.indexOf('Win') > -1;
   me.isMac = navigator.appVersion.indexOf('Mac') > -1;
   me.isUnix = navigator.appVersion.indexOf('X11') > -1;
   me.isLinux = navigator.appVersion.indexOf('Linux') > -1;

   /** Browser detection flags. */
   me.isFirefox = navigator.userAgent.toLowerCase().indexOf('firefox') > -1;
   me.isChromium = navigator.userAgent.toLowerCase().indexOf('chromium') > -1;
   me.isChrome = !!window.chrome;
   me.isIe = !!window.ActiveXObject;

   /**
    * Initialize module.
    * 
    * @param {object} cfg  configuration.
    */
   me.init = function(cfg)
   {
      /**
       * Prevent back navigation.
       */
      history.pushState({}, '', cfg.page);

      /**
       * Prevent back navigation.
       * 
       * @param {event} event object
       */
      window.onpopstate = function(event) 
      {
         if (event.type === 'popstate')
            history.pushState({}, '', cfg.page);
      };

      /**
       * Disable browser context menu.
       * 
       * @returns {boolean}   false = disabled
       *                      true  = enabled
       */
      document.oncontextmenu = function()
      {
         return false;
      };
      
      // save off our client mode type
      p2j.isGui = cfg.isGui;

      // init modules (don't change the order here!)
      p2j.sound.init(cfg);
      p2j.screen.init(cfg);
      
      // TODO: was it OK to move the container centering to the p2j.screen.js init?
      
      p2j.socket.init(cfg);
      p2j.keyboard.init();
      p2j.clipboard_helpers.init(cfg);
      p2j.clipboard.init(cfg);
   };

   /* exports */
   return me;
}());
