/*
** Module   : p2j.keyboard.js
** Abstract : keystroke management
**
** Copyright (c) 2014-2015, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**         Golden Code Development Corporation
**                 CONFIDENTIAL
**
** -#- -I- --Date-- ------------------------------Description----------------------------------
** 001 MAG 20140110 First version.
** 002 MAG 20140228 Add javadoc.
** 003 MAG 20140402 Implements interactive process.
** 004 MAG 20140507 Fix DEL key code for VT100 mode.
** 005 MAG 20140701 Use CTRL-C and CTRL-V to copy/paste.
** 006 GES 20150324 Moved this class to a common resource base and made it generic (so that it
**                  can be used for both ChUI and GUI web clients.
*/

p2j.keyboard = (function()
{
   /** Keyboard object which will be returned from this initialization function. */
   var me = {};    
   
   /** Interactive child process mode if true, normal 4GL compatibility mode if false. */
   me.vt100 = false;
   
   /** Track escape key status. */
   var escape = false;                     
   
   /** Track alt char status. */
   var altChar = false;

   /**
    * Key press event.
    * 
    * @param {event} evt  event object.
    */
   var onkeypress = function(evt)
   {
      var key = evt.charCode;

       // prevent Firefox keypress on CTRL_C and CTRL-V
      if (p2j.clipboard.enabled && p2j.isFirefox && evt.ctrlKey)
      {
         if (key === 99 || key === 118)
         {
            return;
         }
      }
     
      if (key !== 0)
      {         
         if (me.vt100)
         {
            sendKeyPair(0, evt.charCode, evt);
         }
         else
         {
            sendKey(key, evt);               
         }
      }         
   };
              
   /**
    * Key up event.
    * 
    * @param {event} evt  event object.
    */
   var onkeyup = function(evt)
   {
      if (evt.keyCode === keys.ALT && evt.location === evt.DOM_KEY_LOCATION_RIGHT)
      {
         altChar = false;
      }
      
      // stop event propagation
      evt.preventDefault();
      evt.stopImmediatePropagation();
   };

   /**
    * Key down event.
    * 
    * @param {event} evt  event object.
    */
   var onkeydown = function(evt)
   {
      var key = evt.keyCode;
    
      if (altChar || key === keys.CTRL  || key === keys.SHIFT)
      {
         return;
      }

      if (key === keys.ALT)
      {
         if (evt.location === evt.DOM_KEY_LOCATION_RIGHT)
         {
            altChar = evt.ctrlKey;                  
         }
         return;
      }
      
      // in interactive child process mode handle as a key pair 
      if (me.vt100)
      {
         doVT100(key, evt);
         return;
      }
      
      if (key === keys.ESCAPE)
      {
         evt.preventDefault();
         
         // ESC is used to clear selection
         if (p2j.clipboard.enabled && p2j.clipboard.haveSelection())
         {
            p2j.clipboard.clearSelection();
         }
         else
         {
            escape = !escape;
         }
         
         return;               
      }
                               
      var p4code = -1;
      var charCode = p2j.keymap.mapKey(evt);
                                       
      // handle ALT or ESC
      if (evt.altKey || escape)
      {
         p4code = p2j.keymap.xlateAlt[key];
         
         if (p4code === undefined)
         {
            if (charCode !== -1 && ((!escape && evt.altKey) || (escape && !evt.altKey)))
            {
               p4code = p2j.keymap.processCtrl(evt, charCode);
               
               if(p4code === -1)
               {
                  p4code = charCode;
               }
               
               p4code |= 1024;
            }
            else
            {
               p4code = -1;
            }
         }
         else
         {
            if (!escape && evt.altKey && key === keys.DELETE)
            {
               // special case for ALT+DELETE key combo
               p4code = 512;
            }     
         }    
                    
         escape = false;
      }
      else
      {
         // normal map
         var p4code = p2j.keymap.xlate[key];
         
         if (p4code === undefined)
         {
            // check CTRL
            p4code = p2j.keymap.processCtrl(evt, charCode);
         }
         else
         {
            if (p4code === keys.TAB && evt.shiftKey)
            {
               p4code = 509;
            }                           
         }
      }                                                           

      if (p4code !== -1)
      {
         sendKey(p4code, evt);
      }      
   };
               
   /**
    * Initialize module.
    */
   me.init = function()
   {      
      // init mapping   
      p2j.keymap.init();
      
      // TODO: confirm that this must exist after the on* function definitions
      document.addEventListener('keydown', onkeydown, true);
      document.addEventListener('keypress', onkeypress, true);
      document.addEventListener('keyup', onkeyup, true);
   };
                    
   /**
    * Process key strokes on VT100 mode.
    * 
    * @param {number} key key code
    * @param {event}  evt event object
    */
   var doVT100 = function(key, evt)
   {
      if (evt.ctrlKey)
      {
         // CTRL keys
         var charCode = p2j.keymap.mapKey(evt);
         var ctrlCode = charCode & 0xffbf;
         
         sendKeyPair(0, ctrlCode, evt);
      }
      else if (evt.altKey)
      {
         // ALT keys are not used on VT100
         evt.preventDefault();
         evt.stopImmediatePropagation();
      }
      else
      {
         switch (key)
         {  
            case keys.TAB:
            case keys.ENTER:
            case keys.ESCAPE:
               sendKeyPair(0, key, evt);
               break;
            case keys.DELETE:
               sendKeyPair(0x7f, 0, evt);
               break;
            case keys.BACKSPACE:
            case keys.PAGE_UP:
            case keys.PAGE_DOWN:
            case keys.LEFT_ARROW:
            case keys.RIGHT_ARROW:
            case keys.UP_ARROW:
            case keys.DOWN_ARROW:
            case keys.INSERT:
            case keys.HOME:
            case keys.END:
            case keys.F1:
            case keys.F2:
            case keys.F3:
            case keys.F4:
            case keys.F5:
            case keys.F6:
            case keys.F7:
            case keys.F8:
            case keys.F9:
            case keys.F10:
            case keys.F11:
            case keys.F12:
               sendKeyPair(key, 0, evt);
               break;
         }
      }
   };
                                                                              
   /**
    * Send key.
    * 
    * @param {number} key key code
    * @param {event}  evt event object
    */
   var sendKey = function(key, evt)
   {
      // if have clipboard
      if (p2j.clipboard.enabled)
      {
         // CTRL-C (copy) only when have selection
         if (key === 3 && p2j.clipboard.enabled && p2j.clipboard.haveSelection())
         {
            // prepare clipboard operations on CTRL key down (copies selected text into a
            // hidden input field, focuses that field and selects the text)
            p2j.clipboard.prepareCopy();
            
            // default processing will copy the current selection
            return;
         }
   
         // CTRL-V (paste)
         if (key === 22)
         {
            p2j.clipboard.ctrlv = true;
            
            if (!window.Clipboard)
            {
               // use a hidden input field and select all contents of that field
               p2j.clipboard.preparePaste();
            }
            
            // default processing will overwrite the contents with the clipboard contents and
            // a paste listener in the p2j.clipboard module will read that text and send it to
            // the server
            return;
         }
      }
      
      var data = new Uint8Array(3);
      data[0] = 0x01;
      var c = key;
      data[2] = c & 0xff;
      c = c >> 8;
      data[1] = c & 0xff;
      
      p2j.socket.send(data);
      
      evt.preventDefault();
      evt.stopImmediatePropagation();
   };

   /**
    * Send key code and character code.
    * 
    * @param {number} keyCode key code
    * @param {number} charCode key character code
    * @param {event}  evt event object
    */
   var sendKeyPair = function(keyCode, charCode, evt)
   {
      var data = new Uint8Array(5);
      data[0] = 0x02;
      var c = keyCode;
      data[2] = c & 0xff;
      c = c >> 8;
      data[1] = c & 0xff;
      c = charCode;
      data[4] = c & 0xff;
      c = c >> 8;
      data[3] = c & 0xff;
      
      p2j.socket.send(data);
      
      evt.preventDefault();
      evt.stopImmediatePropagation();
   };
   
   return me;    
}());
