/*
** Module   : PopupOption.java
** Abstract : A popup option associated with a popup menu.
**
** Copyright (c) 2014-2015, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- ---------------------------------Description----------------------------------
** 001 CA  20140806 Created initial version.
** 002 HC  20140911 GUI coordinate support - data types refactoring.
** 003 CA  20140926 Added shared widget configuration support, to allow GUI/ChUI concrete
**                  implementation for the same widget ID.  Refactored the widget configuration
**                  classes: all fields were made public; these classes need to be as dumb as
**                  possible, all logic related to setting/getting a certain field should be at
**                  the widget or at the caller. Refs #2254
** 004 CA  20141021 Improved the drawing and behaviour of the popup menu. 
** 005 MAG 20141029 Added clipping rectangle when drawing. 
** 006 HC  20141021 Implemented window move/position support.
** 007 HC  20150323 Removed business logic from config classes.
** 008 CA  20150507 Draw text centered vertically.
** 009 GES 20150428 Eliminated Swing dependencies, this is generic GUI code!
*/

package com.goldencode.p2j.ui.client.gui;

// this usage of AWT events is intentional, they are being used separately from the
// AWT/Swing event model itself; no other AWT/Swing usage is occurring in this class
import java.awt.event.MouseEvent;

import com.goldencode.p2j.ui.*;
import com.goldencode.p2j.ui.client.*;
import com.goldencode.p2j.ui.client.driver.*;
import com.goldencode.p2j.ui.client.gui.driver.*;
import com.goldencode.p2j.ui.client.widget.*;

/**
 * Definition of a popup option.
 */
class PopupOption
extends AbstractWidget<GuiOutputManager>
implements WidgetWithConfig<BaseConfig>
{
   /** The option's label.  May be {@code null}, when acting as a separator. */
   final String label;
   
   /** The option's action.  May be {@code null}. */
   private final Runnable action;
   
   /** Flag indicating if the option is active or not. */
   private boolean active = false;

   /** The graphics driver. */
   private GuiDriver gd;
   
   /** The option's configuration. */
   protected BaseConfig config;
   
   /**
    * Create a new popup option.
    * 
    * @param    id
    *           The widget's ID.
    * @param    label
    *           The label.
    * @param    action
    *           The action.
    */
   public PopupOption(WidgetId id, String label, Runnable action)
   {
      this.label = label;
      this.action = action;

      this.config = ConfigManager.getInstance().resolveWidgetConfig(id, BaseConfig.class);
      this.gd = (GuiDriver) OutputManager.getDriver();
      
      this.nativeInsets = new NativeInsets(2, 2, 2, 2);
   }
   
   /**
    * Create a new popup option.
    * 
    * @param    label
    *           The label.
    * @param    action
    *           The action.
    */
   public PopupOption(WidgetId id, boolean active, String label, Runnable action)
   {
      this(id, label, action);
      
      this.active = active;
   }

   /**
    * Replacement API for the logic which should exist in the widget's c'tor, but the logic 
    * can't be executed as it depends on the widget being registered into the official registry.
    * <p>
    * This API is called after the widget is created and added to the registry.
    * 
    * @param    id
    *           The widget's ID.
    * @param    cfg
    *           The config used to initialize this widget.
    */
   @Override
   public void initialize(WidgetId id, BaseConfig cfg)
   {
      // no-op
   }

   /**
    * Get widget dimension.
    * 
    * @return  widget dimension.
    */
   @Override
   public Dimension dimension()
   {
      return new Dimension(width(), height());
   }

   /**
    * Draw this popup menu option.
    */
   @Override
   public void draw()
   {
      final NativePoint origin = physicalLocation();
      final NativeDimension dim = physicalDimension();
      final NativeRectangle clip = clipRectangle(origin, dim);
      final WindowGuiImpl window = (WindowGuiImpl) ancestor();
      
      gd.draw(origin, clip, new Runnable()
      {
         @Override
         public void run()
         {
            if (label == null)
            {
               int y = dim.height / 2;
               gd.setColor(gd.getSysColor(SysColor.COLOR_ACTIVEBORDER));
               gd.drawLine(nativeInsets.left, y, dim.width, y);
            }
            else
            {
               gd.setColor(gd.getSysColor(!isEnabled() ? SysColor.COLOR_INACTIVECAPTIONTEXT
                                                       : active ? SysColor.COLOR_MENUTEXT 
                                                                : SysColor.COLOR_MENUHILIGHT));

               int font = window.resolveTitleFont();
               FontDetails<?> f = FontManager.resolveFont((Window<?>) ancestor(),
                                                          font,
                                                          OutputManager.getDriver());
               gd.setGuiFont(f);

               gd.drawStringCentered(label, nativeInsets.left, dim.height);
            }
         }
      });
   }

   /**
    * Get the widget config.
    * 
    * @return   See above.
    */
   @SuppressWarnings("unchecked")
   @Override
   public BaseConfig config()
   {
      return config;
   }
   
   /**
    * The method is called after the configuration associated with 
    * the implementor changes. The parameter points to a reference
    * representing the original configuration state. This reference
    * can be used to detect what configuration fields changed and so
    * optimize any processing related to the configuration change.
    * 
    * @param    beforeUpdate
    */
   @Override
   public void afterConfigUpdate(BaseConfig beforeUpdate)
   {
      super.afterConfigUpdateBase(beforeUpdate);
   }
   
   /**
    * Get widget height.
    *
    * @return  widget height.
    */
   @Override
   public double height()
   {
      final WindowGuiImpl window = (WindowGuiImpl) ancestor();
      int font = window.resolveTitleFont();

      double height = 0.5;

      if (label != null)
      {
         ScreenDriver<?> driver = OutputManager.getDriver();
         CoordinatesConversion c = window.screen().coordinates();
         
         int nh = FontManager.getTextHeight(label, window, font, driver) + 
                  nativeInsets.top + nativeInsets.bottom;
               
         height = c.heightFromNative(nh);
      }

      return height;
   }

   /**
    * Get widget minimum size.
    * 
    * @return  widget minimum size.
    */
   @Override
   public Dimension minimumSize()
   {
      return dimension();
   }

   /**
    * Get widget width.
    *
    * @return  widget width.
    */
   @Override
   public double width()
   {
      final WindowGuiImpl window = (WindowGuiImpl) ancestor();
      ScreenDriver<?> driver = OutputManager.getDriver();
      CoordinatesConversion c = window.screen().coordinates();

      int font = window.resolveTitleFont();

      final double width;
      
      if (label != null)
      {
         
         int nw = FontManager.getTextWidth(label, window, font, driver) +
                  nativeInsets.left + nativeInsets.right;
               
         width = c.widthFromNative(nw);
      }
      else
      {
         width = parent().width() - c.widthFromNative(nativeInsets.left + nativeInsets.right);
      }
      
      return width;
   }
   
   /**
    * Notification of a mouse entered event occurred for this widget.
    * 
    * @param    e
    *           The mouse event.
    */
   @Override
   public void mouseEntered(MouseEvent e)
   {
      if (!isEnabled())
      {
         return;
      }

      this.active = true;
      
      repaint();
   }
   
   /**
    * Notification of a mouse exited event occurred for this widget.
    * 
    * @param    e
    *           The mouse event.
    */
   @Override
   public void mouseExited(MouseEvent e)
   {
      if (!isEnabled())
      {
         return;
      }

      this.active = false;

      repaint();
   }
   
   /**
    * Notification of a mouse clicked event occurred for this widget.
    * 
    * @param    e
    *           The mouse event.
    */
   @Override
   public void mouseClicked(MouseEvent e)
   {
      if (!isEnabled())
      {
         return;
      }

      if (action != null && active)
      {
         action.run();
      }

      PopupMenu menu = (PopupMenu) parent();
      menu.processingOption(false);
   }
   
   /**
    * Notification of a mouse pressed event occurred for this widget.
    * <p>
    * This stops the parent popup menu to process mouse click/press events.
    * 
    * @param    e
    *           The mouse event.
    */
   @Override
   public void mousePressed(MouseEvent e)
   {
      if (!isEnabled())
      {
         return;
      }

      PopupMenu menu = (PopupMenu) parent();
      menu.processingOption(true);
   }
   
   /**
    * Check if the widget implementation can raise legacy, 4GL-style, mouse events.
    * 
    * @return   Always <code>false</code>.
    */
   @Override
   public boolean raiseLegacyMouseEvents()
   {
      return false;
   }
}
